
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Interact with LogicBoxes reseller API",
  "dist_author" => [
    "Robert Stone <drzigman\@cpan.org>"
  ],
  "dist_name" => "WWW-LogicBoxes",
  "dist_version" => "1.8.0",
  "license" => "perl",
  "module_name" => "WWW::LogicBoxes",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Util" => 0,
    "Data::Validate::Domain" => 0,
    "Data::Validate::Email" => 0,
    "Data::Validate::IP" => 0,
    "Data::Validate::URI" => 0,
    "DateTime" => 0,
    "HTTP::Tiny" => 0,
    "JSON" => 0,
    "Moose" => 0,
    "Moose::Role" => 0,
    "Moose::Util::TypeConstraints" => 0,
    "MooseX::Aliases" => 0,
    "MooseX::Params::Validate" => 0,
    "MooseX::StrictConstructor" => 0,
    "MooseX::Types" => 0,
    "MooseX::Types::Moose" => 0,
    "Mozilla::PublicSuffix" => 0,
    "Number::Phone" => 0,
    "Readonly" => 0,
    "Try::Tiny" => 0,
    "URI::Escape" => 0,
    "XML::LibXML::Simple" => 0,
    "namespace::autoclean" => 0,
    "overload" => 0,
    "perl" => "5.01301"
  },
  "test_requires" => {
    "Exporter" => 0,
    "File::Spec" => 0,
    "FindBin" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "List::Util" => 0,
    "Storable" => 0,
    "String::Random" => 0,
    "Test::Deep" => 0,
    "Test::Exception" => 0,
    "Test::MockModule" => 0,
    "Test::Moose::More" => "0.029",
    "Test::More" => 0,
    "Test::Warn" => 0,
    "lib" => 0,
    "perl" => "5.01301"
  }
);


my %fallback_build_requires = (
  "Exporter" => 0,
  "File::Spec" => 0,
  "FindBin" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "List::Util" => 0,
  "Module::Build" => "0.28",
  "Storable" => 0,
  "String::Random" => 0,
  "Test::Deep" => 0,
  "Test::Exception" => 0,
  "Test::MockModule" => 0,
  "Test::Moose::More" => "0.029",
  "Test::More" => 0,
  "Test::Warn" => 0,
  "lib" => 0,
  "perl" => "5.01301"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
